// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_pubchem.module.compounds;

/// PubChem compounds table
///
/// See: https://pubchem.ncbi.nlm.nih.gov/pug_rest/PUG_REST.html
final compoundTable = new TableDescriptor(
    remote: true,
    computed: false,
    localCache: true,
    primaryKey: const PrimaryKey(DataType.integer, PubChemCid.id),
    keys: const [
      const fields.InChI(KeyRelationType.oneToOne),
      const fields.InChIKey(KeyRelationType.oneToOne),
      const fields.CanonicalSmiles(KeyRelationType.oneToOne),
      const fields.IsomericSmiles(KeyRelationType.oneToOne)
    ],
    values: const [
      // Properties
      const fields.MolecularFormula(),
      const fields.MolecularWeight(),
      const fields.XLogP(),
      const fields.ExactMass(),
      const fields.MonoisotopicMass(),
      const fields.TPSA(),
      const fields.Complexity(),
      const fields.ElecticalCharge(),
      const fields.HBondDonorCount(),
      const fields.HBondAcceptorCount(),
      const fields.RotatableBondCount()
    ],
    parameters: new Set<String>.from([]));

/// Translation table to translate DataServer key names into PubChem keys.
const keyDict = const {
  PubChemCid.id: 'cid',
  fields.InChI.id: 'inchi',
  fields.InChIKey.id: 'inchikey',
  fields.CanonicalSmiles.id: 'smiles',
  fields.IsomericSmiles.id: 'smiles'
};

/// Translation table to translate DataServer key names into PubChem properties.
const keyPropDict = const {
  fields.InChI.id: 'InChI',
  fields.InChIKey.id: 'InChIKey',
  fields.CanonicalSmiles.id: 'CanonicalSmiles',
  fields.IsomericSmiles.id: 'IsomericSmiles'
};

/// Translation table to translate DataServer value names into PubChem
/// properties.
const valuePropDict = const {
  fields.MolecularFormula.id: 'MolecularFormula',
  fields.MolecularWeight.id: 'MolecularWeight',
  fields.XLogP.id: 'XLogP',
  fields.ExactMass.id: 'ExactMass',
  fields.MonoisotopicMass.id: 'MonoisotopicMass',
  fields.TPSA.id: 'TPSA',
  fields.Complexity.id: 'Complexity',
  fields.ElecticalCharge.id: 'Charge',
  fields.HBondDonorCount.id: 'HBondDonorCount',
  fields.HBondAcceptorCount.id: 'HBondAcceptorCount',
  fields.RotatableBondCount.id: 'RotatableBondCount'
};
