// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library molview_pubchem.module.compounds;

import 'dart:async';
import 'dart:convert';

import 'package:omnibus/omnibus.dart';
import 'package:omnibus/patterns.dart';
import 'package:molview_messages/dataserver.dart';
import 'package:molview_pubchem/molview_pubchem.dart';
import 'package:http/http.dart' as http;
import 'package:molview_messages/standards.dart' as fields;
import 'package:molview_messages/molview_messages_v0.dart' as v0;

part 'information.dart';
part 'retrieve_data.dart';
part 'find_suggestions.dart';

/// Compounds module global state data
class State extends SimpleState {
  // All PubChem key properties mapped to their index in the data table.
  var keyMap = new Map<String, int>();

  // All PubChem value properties mapped to their index in the data table.
  var valueMap = new Map<String, int>();

  /// Constuctor
  State(Omnibus bus, http.Client httpClient) : super(bus, httpClient) {
    // Generate maps.
    for (var i = 0; i < compoundTable.keys.length; i++) {
      keyMap[keyPropDict[compoundTable.keys[i].dataClass]] = i;
    }
    for (var i = 0; i < compoundTable.values.length; i++) {
      valueMap[valuePropDict[compoundTable.values[i].dataClass]] = i;
    }
  }
}

/// Bootstrap PubChem compound table handlers.
Future main(Omnibus bus, http.Client httpClient) async {
  // Bind handlers and publish table.
  bus.bind(new v0.RetrieveDataHandler(retrieveData, new State(bus, httpClient),
      database: 'pubchem', table: 'compounds'));

  // The PubChem autocomplete API is disabled because Wikidata will be used to
  // autocomplete compound names.
  bus.bind(new v0.FindSearchSuggestionsHandler(
      findSuggestions, new State(bus, httpClient)));

  // Announce compounds table
  await bus.event(new v0.AnnounceTable('pubchem', 'compounds', compoundTable));

  // Announce autocomplete provider
  await bus.event(new v0.AnnounceAutocompleteProvider(pubchemAutocpProvider));
}
