// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_pubchem.module.compounds;

/// Retrieve data from the PubChem compounds table.
Future<v0.DataRetrieved> retrieveData(
    v0.RetrieveData request, State state) async {
  try {
    // Construct request URL.
    var requestUrl = '$pubchemApiRoot/compound';

    if (keyDict.containsKey(request.inkeyClass)) {
      // Add compound selector.
      requestUrl =
          '$requestUrl/${keyDict[request.inkeyClass]}/${request.inkeyValue}';

      // Collect key and value properties that can be passed to the PubChem
      // property endpoint.
      var properties = new List<String>();
      request.keys.forEach((int key) =>
          properties.add(keyPropDict[compoundTable.keys[key].dataClass]));
      request.values.forEach((int value) =>
          properties.add(valuePropDict[compoundTable.values[value].dataClass]));

      // Execute properties request.
      Map<String, dynamic> response = JSON.decode((await state.httpClient
              .get('$requestUrl/property/${properties.join(',')}/json'))
          .body) as Map<String, dynamic>;

      // Return values
      if (response.containsKey('Fault')) {
        // Something went wrong.
        return new v0.DataRetrieved(new List<DataRecord>());
      } else {
        var keys = new Map<int, dynamic>();
        var values = new Map<int, dynamic>();
        var data =
            response['PropertyTable']['Properties'][0] as Map<String, dynamic>;

        data.forEach((String key, dynamic value) {
          if (state.keyMap.containsKey(key)) {
            keys[state.keyMap[key]] = value;
          } else if (state.valueMap.containsKey(key)) {
            values[state.valueMap[key]] = value;
          }
        });

        return new v0.DataRetrieved([new DataRecord(keys, values, {})]);
      }
    } else {
      // Publish error event, throw DataServer error, or send error response.
      // (not yet deciced/implemented)
      return new v0.DataRetrieved(new List<DataRecord>());
    }
  } catch (e) {
    // Error logging is not yet implemented in `molview_messages`.
  }
}
